from __future__ import absolute_import

from base64 import b64encode

from exam import fixture

from sentry.models import ApiKey
from sentry.testutils import APITestCase


class OrganizationProjectsTest(APITestCase):

    @fixture
    def org(self):
        return self.create_organization(owner=self.user, name='baz')

    @fixture
    def team(self):
        return self.create_team(organization=self.org)

    @fixture
    def path(self):
        return u'/api/0/organizations/{}/projects/'.format(self.org.slug)

    def check_valid_response(self, response, expected_projects):
        assert response.status_code == 200, response.content
        assert [project.id for project in expected_projects] == [
            int(project_resp['id']) for project_resp in response.data]

    def test_simple(self):
        self.login_as(user=self.user)
        project = self.create_project(teams=[self.team])

        response = self.client.get(self.path)
        self.check_valid_response(response, [project])
        assert self.client.session['activeorg'] == self.org.slug

    def test_with_stats(self):
        self.login_as(user=self.user)

        projects = [self.create_project(teams=[self.team])]

        response = self.client.get(u'{}?statsPeriod=24h'.format(self.path), format='json')
        self.check_valid_response(response, projects)
        assert response.data[0]['stats']

        response = self.client.get(u'{}?statsPeriod=14d'.format(self.path), format='json')
        self.check_valid_response(response, projects)
        assert response.data[0]['stats']

        response = self.client.get(u'{}?statsPeriod='.format(self.path), format='json')
        self.check_valid_response(response, projects)
        assert 'stats' not in response.data[0]

        response = self.client.get(u'{}?statsPeriod=48h'.format(self.path), format='json')
        assert response.status_code == 400

    def test_search(self):
        self.login_as(user=self.user)
        project = self.create_project(teams=[self.team], name='bar', slug='bar')

        response = self.client.get(u'{}?query=bar'.format(self.path))
        self.check_valid_response(response, [project])

        response = self.client.get(u'{}?query=baz'.format(self.path))
        self.check_valid_response(response, [])

    def test_search_by_ids(self):
        self.login_as(user=self.user)

        project_bar = self.create_project(teams=[self.team], name='bar', slug='bar')
        project_foo = self.create_project(teams=[self.team], name='foo', slug='foo')
        self.create_project(teams=[self.team], name='baz', slug='baz')

        path = u'{}?query=id:{}'.format(self.path, project_foo.id)
        response = self.client.get(path)
        self.check_valid_response(response, [project_foo])

        path = u'{}?query=id:{} id:{}'.format(self.path, project_bar.id, project_foo.id)
        response = self.client.get(path)

        self.check_valid_response(response, [project_bar, project_foo])

    def test_bookmarks_appear_first_across_pages(self):
        self.login_as(user=self.user)

        projects = [self.create_project(teams=[self.team], name=i, slug=i) for i in range(3)]
        projects.sort(key=lambda project: project.slug)

        response = self.client.get(self.path)
        self.check_valid_response(response, [project for project in projects])

        response = self.client.get(self.path + '?per_page=2')
        self.check_valid_response(response, [project for project in projects[:2]])

        self.create_project_bookmark(projects[-1], user=self.user)
        # Move the bookmarked project to the front
        projects.insert(0, projects.pop())
        response = self.client.get(self.path)
        self.check_valid_response(response, [project for project in projects])

        # Make sure that it's at the front when on the second page as well
        response = self.client.get(self.path + '?per_page=2')
        self.check_valid_response(response, [project for project in projects[:2]])

        # Make sure that other user's bookmarks don't interfere with this user
        other_user = self.create_user()
        self.create_project_bookmark(projects[1], user=other_user)
        response = self.client.get(self.path)
        self.check_valid_response(response, [project for project in projects])

    def test_api_key(self):
        key = ApiKey.objects.create(
            organization=self.org,
            scope_list=['org:read'],
        )

        project = self.create_project(teams=[self.team])

        response = self.client.get(
            self.path,
            HTTP_AUTHORIZATION='Basic ' + b64encode(u'{}:'.format(key.key)),
        )
        self.check_valid_response(response, [project])
